# -*- coding: utf-8 -*-
# coding = utf-8
# 接口请求统一返回处理
from fastapi import HTTPException, status
from typing import Any, Dict, List, Union
from ..settings import logger

def response_result(data: Union[List[Any], Dict[str, Any], None] = None, msg: str = '操作成功', code: int = 200, operation: str = '默认操作', **kw) -> Dict[str, Any]:
    """
       封装统一返回结果的函数

       参数:
       data: 返回的数据, 默认是空列表
       msg: 返回的消息, 默认是 '操作成功'
       code: 自定义状态码, 默认是 200
       operation: 操作名称

       返回:
       一个包含 state, data 和 msg 的字典
       """
    if data is None:
        data = []
    if code == 200:
        logger.info(f'操作信息：{operation}，错误结果：{msg}’')
    else:
        logger.error(f'code: {code} 。错误信息：{operation} 。错误提示：{msg}')

    return {
        'state': 200,
        'data': {
            'code': code,
            'result': data or None,
            'msg': msg,
            **kw,
        }
    }

def error_message(e):
    # 如果出现任何异常，例如数据库连接问题或SQL执行问题，将抛出
    # HTTP
    # 500
    # 内部服务器错误
    raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e))
    return response_result(code=500, msg=str(e), operation='系统错误')


# modelss/user.py

from pydantic import BaseModel, SecretStr

class User(BaseModel):
    name: str
    username: str
    password: str

class Login(BaseModel):
    username:  str
    password: str
    verification: str
    IV: str
    AES: str

class infoModel(BaseModel):
    info: object


# utils/crypto/aes
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.backends import default_backend
import base64


# aes解密
def aes_decrypt(encrypted_text, private_key, iv):
    private_key_byte = base64.b64decode(private_key)
    iv_byte_value = base64.b64decode(iv)
    encrypted_text_byte = base64.b64decode(encrypted_text)

    # 创建解密aes对象
    cipher = Cipher(algorithms.AES(private_key_byte), modes.CBC(iv_byte_value), backend=default_backend())
    decryptor = cipher.decryptor()

    try:
        # 解密
        padded_plain_text = decryptor.update(encrypted_text_byte) + decryptor.finalize()
        # 去除解密后的填充
        unpadder = padding.PKCS7(algorithms.AES.block_size).unpadder()
        plain_text = unpadder.update(padded_plain_text) + unpadder.finalize()
        return plain_text.decode('utf-8')
    except ValueError as e:
        print(f'aes解密错误{e}')
        raise







# utils/crypto/rsa
from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.backends import default_backend
import base64
import os

# 配置密钥存储路径
KEYS_DIR = "keys"
private_key_path = os.path.join(KEYS_DIR, "private_key.pem")
public_key_path = os.path.join(KEYS_DIR, "public_key.pem")

os.makedirs(KEYS_DIR, exist_ok=True)
# 生成rsa秘钥对保存
def generate_rsa_key():
    # 生成 rsa 秘钥对
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048
    )
    public_key = private_key.public_key()

    # 将秘钥保存到pem文件中
    with open(private_key_path, 'wb') as private_key_file:
        private_key_file.write(
            private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=serialization.NoEncryption()
            )
        )

    # 保存公钥
    with open(public_key_path, 'wb') as public_key_file:
        public_key_file.write(
            public_key.public_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PublicFormat.SubjectPublicKeyInfo
            )
        )

# 返回私钥，格式是utf8格式
def load_private_key(isStr = False):
    # 加载私钥
    with open(private_key_path, 'rb') as private_key_file:
        private_key_data = private_key_file.read()

    private_key = serialization.load_pem_private_key(
        private_key_data,
        password=None,  # 如果私钥是加密的，需要提供解密密码
        backend=default_backend()
    )

    if isStr:
        # 将私钥序列化为 PEM 格式的字节数据
        pem_data = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()
        )
        return pem_data.decode('utf-8').strip().replace('-----BEGIN PUBLIC KEY-----\n', '').replace('\n-----END PUBLIC KEY-----', '')
    else:
        return private_key


# 返回公钥，格式utf8
def load_public_key(isStr = False):
    # 加载公钥
    with open(public_key_path, 'rb') as public_key_file:
        public_key = serialization.load_pem_public_key(
            public_key_file.read()
        )
    if isStr:
        # 将公钥序列化为 PEM 格式的字节数据
        pem_data = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        )
        return pem_data.decode('utf-8').strip().replace('-----BEGIN PUBLIC KEY-----\n', '').replace('\n-----END PUBLIC KEY-----', '')
    else:
        return public_key

# 私钥解码
def encrypted_decode_key(password):
    # 获取私钥
    private_key = load_private_key()
    try:
        # 解密 AES 密钥
        decrypted_aes_key = private_key.decrypt(
            base64.b64decode(password),
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA1()),
                algorithm=hashes.SHA1(),
                label=None
            )
        )
        return decrypted_aes_key
    except ValueError as e:
        print("Decryption failed:", e)
        raise





# coding = utf-8
# app/auth/routes.py
from fastapi import APIRouter, Depends
from aiomysql import Connection
from personage_net.db.connection import get_db_conn
from personage_net.models.user import User, Login
from personage_net.utils.response import response_result, error_message
from personage_net.utils.crypto.rsa import encrypted_decode_key
from personage_net.utils.crypto.aes import aes_decrypt
from personage_net.utils.crypto.bcrypt import check_password, hashed_password
router = APIRouter()

@router.get('/fast/getUserlist')
async def read_user_list(conn=Depends(get_db_conn)):
    async with conn.cursor() as cursor:
        await cursor.execute("SELECT * FROM my_project.userinfo")
        result = await cursor.fetchall()
        if not result:
            return response_result(code=200, msg='查询成功', operation='查询用户列表')
        data = [{'name': row[1], 'id': row[0], 'username': row[2]} for row in result]
        return response_result(code=200, msg='查询成功', operation='查询用户列表', data=data)

@router.post('/fast/createUser')
async def save_user_info(user_info: User, conn: Connection = Depends(get_db_conn)):
    try:
        async with conn.cursor() as cursor:
            await cursor.execute(f"SELECT * from my_projct.useringo where username = {user_info.username}")
            result = await cursor.fetchall()
            if not result:
                await cursor.execute("INSERT INTO my_project.userinfo (name, username, password) VALUES (%s, %s, %s)",
                                     (user_info.name, user_info.username, user_info.password))
                return response_result(code=200, msg='用户创建成功', operation='创建用户')
            return response_result(code=1000, msg='用户名存在', operation='创建用户')
    except Exception as e:
        error_message(e)

@router.post('/fast/login')
async def user_login(LoginInfo: Login, conn: Connection = Depends(get_db_conn) ):
    # rsa私钥进行解密aes私钥
    aes_private_key = encrypted_decode_key(LoginInfo.AES)


    # aes跟IV反解密码跟账户
    aes_dispose_password = aes_decrypt(LoginInfo.password, aes_private_key, LoginInfo.IV)
    username = aes_decrypt(LoginInfo.username, aes_private_key, LoginInfo.IV)

    async with conn.cursor() as cursor:
        await cursor.execute("SELECT password FROM my_project.userinfo u where username = %s", username)
        result = await cursor.fetchone()
        if not result:
            return response_result(code=10002, msg='用户不存在', operation='用户登录')
        if check_password(result[0], aes_dispose_password):
            return response_result(code=200, msg='登录成功', operation='用户登录', userInfo = {"token":'111122'})
        else:
            return response_result(code=10001, msg='密码或用户名有误，请检查后重新输入', operation='用户登录')




# db/connection.py
import aiomysql
from typing import Optional

db_pool: Optional[aiomysql.Pool] = None

async def init_db_pool(host: str, port: int, user: str, password: str, db: str):
    global db_pool
    db_pool = await aiomysql.create_pool(
        host=host,
        port=port,
        user=user,
        password=password,
        db=db,
        autocommit=True
    )

async def close_db_pool():
    global db_pool
    if db_pool:
        db_pool.close()
        await db_pool.wait_closed()

async def get_db_conn():
    global db_pool
    if db_pool:
       return await db_pool.acquire()
    else:
        raise RuntimeError("Database connection pool is not initialized.")




# coding = utf-8
# app/utils/routes.py
from fastapi import APIRouter
from personage_net.utils.crypto.rsa import generate_rsa_key ,load_public_key, private_key_path, public_key_path
from personage_net.utils.response import response_result, error_message
import os

router = APIRouter()

# 获取rsa的公钥
@router.get('/fast/getRsaKey')
async def get_rsa_key():
    try:
        # 检查 RSA 密钥文件是否存在
        if not (os.path.exists(private_key_path) and os.path.exists(public_key_path)):
            # 如果密钥文件不存在，则生成新的 RSA 密钥
            generate_rsa_key()
        # 加载 RSA 密钥
        public_key = load_public_key(True)

        # 返回成功响应，包含公钥
        return response_result(code=200, msg='操作成功', operation='查询rsa公钥', key=public_key)
    except Exception as e:
        # 捕获所有异常并处理
        error_message(e)


import logging

def setup_logging():
    #  ��������־��¼��
    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    # ������־д��error.log �ļ�
    error_handler = logging.FileHandler('logs/error.log')
    error_handler.setLevel(logging.ERROR)
    error_formatter = logging.Formatter('%(asctime)s %(levelname)s:%(message)s', datefmt='%Y-%m-%d %H:%M:%S')
    error_handler.setFormatter(error_formatter)

    # ������־ д��operation.log �ļ�
    operation_handler = logging.FileHandler('logs/operation.log')
    operation_handler.setLevel(logging.INFO)
    operation_formatter = logging.Formatter('%(asctime)s %(levelname)s:%(message)s', datefmt='%Y-%m-%d %H:%M:%S')
    operation_handler.setFormatter(operation_formatter)

    root_logger.addHandler(error_handler)
    root_logger.addHandler(operation_handler)

# ��ʼ����־����
setup_logging()

logger = logging.getLogger(__name__)


# coding = utf-8
# app/index.py
from fastapi import FastAPI
from .auth.routes import router as auto_router
from .utils.routes import router as utils_router
from personage_net.db.connection import init_db_pool, close_db_pool
import configparser
import os

def create_app() -> FastAPI:
    app = FastAPI()

    # ��ȡ��ǰ����Ŀ¼
    current_dir = os.path.dirname(os.path.abspath(__file__))
    config_path = os.path.join(current_dir, '../db/config.ini')

    # ���� configparser ����
    db_config = configparser.ConfigParser()
    db_config.read(config_path)

    DB_HOST = db_config['DATABASE']['DB_HOST']
    DB_PORT = db_config.getint('DATABASE', 'DB_PORT')
    DB_USER = db_config['DATABASE']['DB_USER']
    DB_PASSWORD = db_config['DATABASE']['DB_PASSWORD']
    DB_NAME = db_config['DATABASE']['DB_NAME']

    # �����¼�
    @app.on_event('startup')
    async def startup():
        await init_db_pool(host=DB_HOST, port=DB_PORT, user=DB_USER, password=DB_PASSWORD, db=DB_NAME)

    # �ر��¼�
    @app.on_event('shutdown')
    async def shutdown():
        await close_db_pool()

    # ע��·��
    app.include_router(auto_router, tags=['auth'])
    app.include_router(utils_router, tags=['utils'])

    return app



# utils/crypto/bcrypt
import bcrypt

# 密码哈希
def hashed_password(password: str) -> str:
    """
    加密过程
    :param password: str
    :return: str
    """
    # 生成盐值
    salt = bcrypt.gensalt()

    # 使用盐值进和密码生成哈希
    hashed_password = bcrypt.hashpw(password.encode('utf-8'), salt)
    return hashed_password.decode('utf-8')

# 哈希值比对
def check_password(stored_hashed_password: str, password: str) -> bool:
    """
       验证密码
       :param stored_hashed_password: str, 存储的哈希密码
       :param password: str, 用户输入的密码
       :return: bool, 验证结果
       """
    # 将存储的哈希密码转换为字节形式
    stored_hashed_password_bytes = stored_hashed_password.encode('utf-8')
    # 使用存储的哈希密码进行验证
    return bcrypt.checkpw(password.encode('utf-8'), stored_hashed_password_bytes)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open('requirements.txt', encoding='utf-8') as f:
    install_requires = f.read().splitlines()

setup(
    name='personage-net',
    version='1.0.4',
    author='tangjiadong',
    author_email='958999498@qq.com',
    description='一个在线工具',
    long_description=open("README.md").read(),  # 从 README 文件中读取长描述
    long_description_content_type="text/markdown",
    url='https://github.com/TungTJD/personage_net.git',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[  # 分类器，用于在PyPI上显示项目的相关信息
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.12',  # Python版本要求
    install_requires=install_requires,
)
